/*
 * Copyright 2009 SpringSource
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.springsource.bundlor.blint.support.contributors;

import static org.junit.Assert.assertEquals;

import java.util.Set;

import org.junit.Test;

import com.springsource.bundlor.util.SimpleManifestContents;
import com.springsource.util.parser.manifest.ManifestContents;

public class VersionedImportsRangeValidatorTests {

    private final VersionedImportsRangeValidator validator = new VersionedImportsRangeValidator();

    @Test
    public void valid() {
        ManifestContents manifest = new SimpleManifestContents();
        manifest.getMainAttributes().put("Import-Package", "foo;version=\"[1,2)\",bar;version=\"3\",baz;version=\"[4,5]\"");
        Set<String> warnings = this.validator.validate(manifest);
        assertEquals(0, warnings.size());
    }

    @Test
    public void reversed() {
        ManifestContents manifest = new SimpleManifestContents();
        manifest.getMainAttributes().put("Import-Package", "foo;version=\"[2,1)\",bar;version=\"[2,1]\",baz;version=\"(2,1)\"");
        Set<String> warnings = this.validator.validate(manifest);
        assertEquals(3, warnings.size());
    }

    @Test
    public void exclusive() {
        ManifestContents manifest = new SimpleManifestContents();
        manifest.getMainAttributes().put("Import-Package", "foo;version=\"(1,2)\",bar;version=\"[1,1]\",baz;version=\"(1,1)\"");
        Set<String> warnings = this.validator.validate(manifest);
        assertEquals(1, warnings.size());
    }
    
    @Test
    public void missingComma() {
        ManifestContents manifest = new SimpleManifestContents();
        manifest.getMainAttributes().put("Import-Package", "foo;version=\"(1,2)\",bar;version=\"[1 1]\",baz;version=\"(1,2)\"");
        Set<String> warnings = this.validator.validate(manifest);
        assertEquals(1, warnings.size());
    }

}
